#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "auxiliar.h"
#include "SDL_net.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activeMAP.h"
#include "activemog2.h"
#include "mog2multiplayer.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BListWnd.h"
#include "BMultiListWnd.h"
#include "GamePreviewDlg.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BFileDlg.h"

#include "GameWnd.h"
#include "GameDebugWnd.h"
#include "PlayerConfigWnd.h"
#include "ScreenConfigDlg.h"
#include "NetworkConfigDlg.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int extra_flags;


BGamePreviewDlg::BGamePreviewDlg(int x,int y,int dx,int dy,MOG2 *g,MAP *m,ROOM *r,TTF_Font *font) : BActiveWindow(x,y,dx,dy,font)
{
	BButton *b;

	b=new BButton(20,dy-40,80,12,"PREVIEW",1);
	buttons.Add(b);
	b=new BButton(20,dy-20,80,12,"BACK",2);
	buttons.Add(b);

	{
		List<OBJECT> l;
		OBJECT *o;

		l.Instance(g->objects);
		l.Rewind();
		while(l.Iterate(o)) {
			if (o->constitution==CONS_CHARACTER) characters.Add(o);
		} /* while */ 
	}

	game=g;
	map=m;
	room=r;

	gameinfo=new NWMOG2GameInfo(game);

	debug=false;
	show_debugmsg=true;
	show_fps=true;

	b=new BButton(10,30,80,12,"ADD",3);
	buttons.Add(b);
	b=new BButton(10,50,80,12,"SUB",4);
	buttons.Add(b);

	bdebug=new BButton(20,dy-60,10,10,"",0);
	bnormal=new BButton(120,dy-60,10,10,"",0);
	bload=new BButton(120,dy-40,10,10,"",0);
	bsave=new BButton(120,dy-20,10,10,"",0);
	game_mode=0;
	load_file=new char[strlen("default.rpl")+1];
	strcpy(load_file,"default.rpl");
	save_file=new char[strlen("default.rpl")+1];
	strcpy(save_file,"default.rpl");

	b=new BButton(10,90,80,12,"NETWORK",5);
	buttons.Add(b);

	b=new BButton(10,110,80,12,"SCREEN",6);
	buttons.Add(b);

	/* Add a player: */ 
	button_pressed(3);

	rebuild_split=true;


} /* BGamePreviewDlg::BGamePreviewDlg */ 


BGamePreviewDlg::~BGamePreviewDlg()
{
	delete []save_file;
	delete []load_file;
	delete bsave;
	delete bload;
	delete bnormal;
	delete bdebug;

	delete gameinfo;

	while(!characters.EmptyP()) characters.ExtractIni();

} /* BGamePreviewDlg::~BGamePreviewDlg */ 


bool BGamePreviewDlg::cycle(int stx,int sty)
{
	bool retval;

	retval=BWindow::cycle(stx,sty);

	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		if (local_mouse_x>=bdebug->m_x && local_mouse_x<bdebug->m_x+bdebug->m_dx &&
			local_mouse_y>=bdebug->m_y && local_mouse_y<bdebug->m_y+bdebug->m_dy) {
			if (debug) debug=false;
				  else debug=true;
			rebuild_split=true;
			set_dirty();
		} /* if */ 

		if (local_mouse_x>=bnormal->m_x && local_mouse_x<bnormal->m_x+bnormal->m_dx &&
			local_mouse_y>=bnormal->m_y && local_mouse_y<bnormal->m_y+bnormal->m_dy) {
			game_mode=0;
			set_dirty();
		} /* if */ 

		if (local_mouse_x>=bload->m_x && local_mouse_x<bload->m_x+bload->m_dx &&
			local_mouse_y>=bload->m_y && local_mouse_y<bload->m_y+bload->m_dy) {
			game_mode=1;
			set_dirty();
		} /* if */ 

		if (local_mouse_x>=bsave->m_x && local_mouse_x<bsave->m_x+bsave->m_dx &&
			local_mouse_y>=bsave->m_y && local_mouse_y<bsave->m_y+bsave->m_dy) {
			game_mode=2;
			set_dirty();
		} /* if */ 

		if (local_mouse_x>=220 && local_mouse_x<get_text_width(load_file,font)+220 &&
			local_mouse_y>=old_dy-40 && local_mouse_y<=TTF_FontHeight(font)+old_dy-40) {
			char tmp[256];

			strcpy(tmp,load_file);
			BFileDlg *dlg=new BFileDlg(30,30,200,200,"Choose a replay to load","Select",tmp,font);

			dlg->Execute();
			delete dlg;

			if (tmp[0]!=0) {
				delete load_file;
				load_file=new char[strlen(tmp)+1];
				strcpy(load_file,tmp);
			} /* if */ 
			set_dirty();
		} /* if */ 

		if (local_mouse_x>=220 && local_mouse_x<get_text_width(save_file,font)+220 &&
			local_mouse_y>=old_dy-20 && local_mouse_y<=TTF_FontHeight(font)+old_dy-20) {
			char tmp[256];

			strcpy(tmp,save_file);
			BFileDlg *dlg=new BFileDlg(30,30,200,200,"Choose a file to save replay","Select",tmp,font);

			dlg->Execute();
			delete dlg;

			if (tmp[0]!=0) {
				delete save_file;
				save_file=new char[strlen(tmp)+1];
				strcpy(save_file,tmp);
			} /* if */ 
			set_dirty();
		} /* if */ 
	
	} /* if */ 

	return retval;
} /* BGamePreviewDlg::cycle */ 



void BGamePreviewDlg::contens_size(int *dx,int *dy)
{
	BActiveWindow::contens_size(dx,dy);

	if (old_dx>*dx) *dx=old_dx;
	if (old_dy>*dy) *dy=old_dy;

	if (get_text_width(save_file,font)+220>*dx) *dx=get_text_width(save_file,font)+220;
	if (get_text_width(load_file,font)+220>*dx) *dx=get_text_width(load_file,font)+220;
} /* BGamePreviewDlg::contens_size */ 


void BGamePreviewDlg::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_Rect r;

	SDL_FillRect(sfc,0,0x0000007f);

	r.x=1-stx;
	r.y=1-sty;
	r.w=dx-2;
	r.h=dy-2;
	SDL_FillRect(sfc,&r,0x000000ff);

	BActiveWindow::draw_contens(sfc,stx,sty,dx,dy);

	print_left("PLAYERS:",font,255,255,255,sfc,10-stx,10-sty);

	local_draw_button(sfc,bdebug,stx,sty,debug); 
	print_left("DEBUG",font,255,255,255,sfc,40-stx,dy-60-sty);

	local_draw_button(sfc,bnormal,stx,sty,(game_mode==0 ? true : false));
	local_draw_button(sfc,bload,stx,sty,(game_mode==1 ? true : false));
	local_draw_button(sfc,bsave,stx,sty,(game_mode==2 ? true : false));
	print_left("NORMAL",font,255,255,255,sfc,140-stx,dy-60-sty);
	print_left("LOAD REPLAY",font,255,255,255,sfc,140-stx,dy-40-sty);
	print_left("SAVE REPLAY",font,255,255,255,sfc,140-stx,dy-20-sty);

	print_left(load_file,font,255,255,255,sfc,220-stx,dy-40-sty);
	print_left(save_file,font,255,255,255,sfc,220-stx,dy-20-sty);
} /* BGamePreviewDlg::draw_contens */ 


bool BGamePreviewDlg::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		if (gameinfo->get_local_nplayers()>0) {

			if (rebuild_split) {
				BScreenConfigDlg *dlg;

				dlg=new BScreenConfigDlg(0,0,320,220,&gameinfo->local->players,&gameinfo->screen_x,&gameinfo->screen_y,&gameinfo->render,&show_debugmsg,&show_fps,debug,font);
				dlg->build_basic_split();
				delete dlg;
			} /* if */ 

			int old_extra_flags=extra_flags;
			if (gameinfo->render==0) {
				/* SOFTWARE RENDERING */ 
				extra_flags=0;
			} else {
				/* HARDWARE RENDERING */ 
				extra_flags=SDL_HWSURFACE|SDL_DOUBLEBUF;
			} /* if */ 

			if (debug) {
				GameDebugWindow *wnd=new GameDebugWindow(gameinfo->screen_x,gameinfo->screen_y,font,game,game_mode,show_fps,(game_mode==1 ? load_file:save_file),&gameinfo->local->players);
				wnd->Execute();
				delete wnd;	
				return true;
			} else {
				GameWindow *wnd=new GameWindow(gameinfo->screen_x,gameinfo->screen_y,font,game,game_mode,show_debugmsg,show_fps,(game_mode==1 ? load_file:save_file),&gameinfo->local->players);
				wnd->Execute();
				delete wnd;		
				return true;
			} /* if */ 

			extra_flags=old_extra_flags;
		} /* if */ 
		break;

	case 2:
		return false;
		break;

	case 3:
		/* ADD ONE PLAYER: */ 
		{
			NWMOG2PlayerInfo *pc;
			BButton *b;

			pc=new NWMOG2PlayerInfo();
			pc->avatar=characters[0];
			pc->vp_area.w=1;
			if (gameinfo->get_local_nplayers()==0 ||
				gameinfo->get_local_nplayers()>1) {
				pc->left_key=SDLK_LEFT;
				pc->right_key=SDLK_RIGHT;
				pc->up_key=SDLK_UP;
				pc->down_key=SDLK_DOWN;
				pc->fire1_key=SDLK_SPACE;
				pc->fire2_key=SDLK_m;
				pc->fire3_key=SDLK_UNKNOWN;
				pc->fire4_key=SDLK_UNKNOWN;
			} else {
				pc->left_key=SDLK_a;
				pc->right_key=SDLK_d;
				pc->up_key=SDLK_w;
				pc->down_key=SDLK_s;
				pc->fire1_key=SDLK_LSHIFT;
				pc->fire2_key=SDLK_LCTRL;
				pc->fire3_key=SDLK_UNKNOWN;
				pc->fire4_key=SDLK_UNKNOWN;
			} /* if */ 
			pc->start_map=map;
			pc->start_room=room;
			pc->start_interface=0;
			pc->player_ID=-1;
			sprintf(pc->name,"player %i",gameinfo->get_local_nplayers()+1);

			b=new BButton(110+(gameinfo->get_local_nplayers()/10)*100,20+(gameinfo->get_local_nplayers()%10)*20,80,12,pc->name,10+gameinfo->get_local_nplayers());
			buttons.Add(b);

			gameinfo->local->players.Add(pc);
			rebuild_split=true;

			set_dirty();
		}
		
		break;

	case 4:
		/* SUB ONE PLAYER: */ 
		{
			NWMOG2PlayerInfo *pc;
			BButton *b;

			if (!gameinfo->local->players.EmptyP()) {
				b=buttons.Extract();
				delete b;
				pc=gameinfo->local->players.Extract();
				delete pc;
			} /* if */ 

			rebuild_split=true;

			set_dirty();
		}

		break;

	case 5:
		/* NETWORK: */ 
		if (game_mode!=1) {
			int x,y;
			BNetworkConfigDlg *dlg;

			if (rebuild_split) {
				BScreenConfigDlg *dlg;

				dlg=new BScreenConfigDlg(0,0,320,220,&gameinfo->local->players,&gameinfo->screen_x,&gameinfo->screen_y,&gameinfo->render,&show_debugmsg,&show_fps,debug,font);
				dlg->build_basic_split();
				delete dlg;
			} /* if */ 

			new_window_coordinates(400,240,&x,&y);

			dlg=new BNetworkConfigDlg(x,y,400,240,game_mode,(game_mode==1 ? load_file:save_file),gameinfo,&characters,font);
			dlg->Execute();

			delete dlg;
		} else {
			draw_message("You cannot REPLAY a network game on-line!");
		} /* if */ 
		break;

	case 6:
		/* SCREEN: */ 
		{
			int x,y;
			BScreenConfigDlg *dlg;

			new_window_coordinates(320,190,&x,&y);

			dlg=new BScreenConfigDlg(x,y,320,220,&gameinfo->local->players,&gameinfo->screen_x,&gameinfo->screen_y,&gameinfo->render,&show_debugmsg,&show_fps,debug,font);
			dlg->Execute();

			rebuild_split=false;

			delete dlg;
		}
		break;

	default:
		/* Pressed player ID-10: */ 
		{
			int x,y;
			BPlayerConfigDlg *dlg;

			new_window_coordinates(450,190,&x,&y);

			dlg=new BPlayerConfigDlg(x,y,450,190,gameinfo->local->players[ID-10],&characters,&game->interfaces,font);
			dlg->Execute();

			delete dlg;
		}
		break;
	} /* switch */ 

	return true;
} /* BGamePreviewDlg::button_pressed */ 
